/* vim:set sts=4 ts=4 noexpandtab: */
/*
 * This confidential and proprietary software may be used only as
 * authorised by a licensing agreement from ARM Limited
 * (C) COPYRIGHT 2009 ARM Limited
 * ALL RIGHTS RESERVED
 * The entire notice above must be reproduced on all authorised
 * copies and copies may only be made to the extent permitted
 * by a licensing agreement from ARM Limited.
 */

#ifndef __3d_h
#define __3d_h

#include <GLES2/gl2.h>

/* Tri strips, so quads are in this order:
 *
 * 2 ----- 3
 * | \     |
 * |   \   |
 * |     \ |
 * 0 ----- 1
 */
static const float aFloorVertex[] =
{
  -6.0f, -0.5f,  6.0f,
   6.0f, -0.5f,  6.0f,
  -6.0f, -0.5f, -6.0f,
   6.0f, -0.5f, -6.0f,
};

static const float aFloorColor[] =
{
  0.4f, 0.3f,  0.2f,
  0.4f, 0.3f,  0.2f,
  0.4f, 0.3f,  0.2f,
  0.4f, 0.3f,  0.2f,
};

static const float aFloorNormal[] =
{
  0.0f, 1.0f,  0.0f,
  0.0f, 1.0f,  0.0f,
  0.0f, 1.0f,  0.0f,
  0.0f, 1.0f,  0.0f,
};

static const GLushort aFloorIndex[] =
{
  0, 1, 2, 1, 3, 2
};

/* Tri strips, so quads are in this order:
 *
 * 2 ----- 3
 * | \     |
 * |   \   | 4 - 5
 * |     \ | | / |
 * 0 ----- 1 6 - 7
 */
static const float aCubeVertex[] =
{
  //FRONT
  -0.5f, -0.5f,  0.5f, /* 0 */
   0.5f, -0.5f,  0.5f, /* 1 */
  -0.5f,  0.5f,  0.5f, /* 2 */
   0.5f, -0.5f,  0.5f, /* 1 */
   0.5f,  0.5f,  0.5f, /* 3 */
  -0.5f,  0.5f,  0.5f, /* 2 */

  // BACK
  -0.5f, -0.5f, -0.5f, /* 6 */
  -0.5f,  0.5f, -0.5f, /* 4 */
   0.5f,  0.5f, -0.5f, /* 5 */
  -0.5f, -0.5f, -0.5f, /* 6 */
   0.5f,  0.5f, -0.5f, /* 5 */
   0.5f, -0.5f, -0.5f, /* 7 */

  // TOP
  -0.5f,  0.5f,  0.5f, /* 2 */
   0.5f,  0.5f,  0.5f, /* 3 */
   0.5f,  0.5f, -0.5f, /* 5 */
  -0.5f,  0.5f,  0.5f, /* 2 */
   0.5f,  0.5f, -0.5f, /* 5 */
  -0.5f,  0.5f, -0.5f, /* 4 */

  // BOTTOM
  -0.5f, -0.5f,  0.5f, /* 0 */
   0.5f, -0.5f,  0.5f, /* 1 */
   0.5f, -0.5f, -0.5f, /* 7 */
  -0.5f, -0.5f,  0.5f, /* 0 */
   0.5f, -0.5f, -0.5f, /* 7 */
  -0.5f, -0.5f, -0.5f, /* 6 */

  // LEFT
  -0.5f, -0.5f, -0.5f, /* 6 */
  -0.5f, -0.5f,  0.5f, /* 0 */
  -0.5f,  0.5f,  0.5f, /* 2 */
  -0.5f, -0.5f, -0.5f, /* 6 */
  -0.5f,  0.5f,  0.5f, /* 2 */
  -0.5f,  0.5f, -0.5f, /* 4 */

  // RIGHT
   0.5f, -0.5f,  0.5f, /* 1 */
   0.5f, -0.5f, -0.5f, /* 7 */
   0.5f,  0.5f, -0.5f, /* 5 */
   0.5f, -0.5f,  0.5f, /* 1 */
   0.5f,  0.5f, -0.5f, /* 5 */
   0.5f,  0.5f,  0.5f, /* 3 */
};

static const float aCubeNormal[] =
{
  //FRONT
   0.0f,  0.0f,  1.0f, /* 0 */
   0.0f,  0.0f,  1.0f, /* 1 */
   0.0f,  0.0f,  1.0f, /* 2 */
   0.0f,  0.0f,  1.0f, /* 1 */
   0.0f,  0.0f,  1.0f, /* 3 */
   0.0f,  0.0f,  1.0f, /* 2 */

  // BACK
   0.0f,  0.0f, -1.0f, /* 6 */
   0.0f,  0.0f, -1.0f, /* 4 */
   0.0f,  0.0f, -1.0f, /* 5 */
   0.0f,  0.0f, -1.0f, /* 6 */
   0.0f,  0.0f, -1.0f, /* 5 */
   0.0f,  0.0f, -1.0f, /* 7 */

  // TOP
   0.0f,  1.0f,  0.0f, /* 2 */
   0.0f,  1.0f,  0.0f, /* 3 */
   0.0f,  1.0f,  0.0f, /* 5 */
   0.0f,  1.0f,  0.0f, /* 2 */
   0.0f,  1.0f,  0.0f, /* 5 */
   0.0f,  1.0f,  0.0f, /* 4 */

  // BOTTOM
   0.0f, -1.0f,  0.0f, /* 0 */
   0.0f, -1.0f,  0.0f, /* 1 */
   0.0f, -1.0f,  0.0f, /* 7 */
   0.0f, -1.0f,  0.0f, /* 0 */
   0.0f, -1.0f,  0.0f, /* 7 */
   0.0f, -1.0f,  0.0f, /* 6 */

  // LEFT
  -1.0f,  0.0f,  0.0f, /* 6 */
  -1.0f,  0.0f,  0.0f, /* 0 */
  -1.0f,  0.0f,  0.0f, /* 2 */
  -1.0f,  0.0f,  0.0f, /* 6 */
  -1.0f,  0.0f,  0.0f, /* 2 */
  -1.0f,  0.0f,  0.0f, /* 4 */

  // RIGHT
   1.0f,  0.0f,  0.0f, /* 1 */
   1.0f,  0.0f,  0.0f, /* 7 */
   1.0f,  0.0f,  0.0f, /* 5 */
   1.0f,  0.0f,  0.0f, /* 1 */
   1.0f,  0.0f,  0.0f, /* 5 */
   1.0f,  0.0f,  0.0f, /* 3 */
};

static const float aCubeColor[] =
{
  //FRONT
  0.0f, 0.0f, 0.0f, 1.0f, /* 0 */
  1.0f, 0.0f, 0.0f, 1.0f, /* 1 */
  0.0f, 1.0f, 0.0f, 1.0f, /* 2 */
  1.0f, 0.0f, 0.0f, 1.0f, /* 1 */
  1.0f, 1.0f, 0.0f, 1.0f, /* 3 */
  0.0f, 1.0f, 0.0f, 1.0f, /* 2 */

  // BACK
  0.0f, 1.0f, 1.0f, 1.0f, /* 6 */
  0.0f, 0.0f, 1.0f, 1.0f, /* 4 */
  1.0f, 0.0f, 1.0f, 1.0f, /* 5 */
  0.0f, 1.0f, 1.0f, 1.0f, /* 6 */
  1.0f, 0.0f, 1.0f, 1.0f, /* 5 */
  1.0f, 1.0f, 1.0f, 1.0f, /* 7 */

  // TOP
  0.0f, 1.0f, 0.0f, 1.0f, /* 2 */
  1.0f, 1.0f, 0.0f, 1.0f, /* 3 */
  1.0f, 0.0f, 1.0f, 1.0f, /* 5 */
  0.0f, 1.0f, 0.0f, 1.0f, /* 2 */
  1.0f, 0.0f, 1.0f, 1.0f, /* 5 */
  0.0f, 0.0f, 1.0f, 1.0f, /* 4 */

  // BOTTOM
  0.0f, 0.0f, 0.0f, 1.0f, /* 0 */
  1.0f, 0.0f, 0.0f, 1.0f, /* 1 */
  1.0f, 1.0f, 1.0f, 1.0f, /* 7 */
  0.0f, 0.0f, 0.0f, 1.0f, /* 0 */
  1.0f, 1.0f, 1.0f, 1.0f, /* 7 */
  0.0f, 1.0f, 1.0f, 1.0f, /* 6 */

  // LEFT
  0.0f, 1.0f, 1.0f, 1.0f, /* 6 */
  0.0f, 0.0f, 0.0f, 1.0f, /* 0 */
  0.0f, 1.0f, 0.0f, 1.0f, /* 2 */
  0.0f, 1.0f, 1.0f, 1.0f, /* 6 */
  0.0f, 1.0f, 0.0f, 1.0f, /* 2 */
  0.0f, 0.0f, 1.0f, 1.0f, /* 4 */

  // RIGHT
  1.0f, 0.0f, 0.0f, 1.0f, /* 1 */
  1.0f, 1.0f, 1.0f, 1.0f, /* 7 */
  1.0f, 0.0f, 1.0f, 1.0f, /* 5 */
  1.0f, 0.0f, 0.0f, 1.0f, /* 1 */
  1.0f, 0.0f, 1.0f, 1.0f, /* 5 */
  1.0f, 1.0f, 0.0f, 1.0f, /* 3 */
};

static const float aCubeTexCoord[] =
{
  //FRONT
  0.0, 0.0, /* 0 */
  1.0, 0.0, /* 1 */
  0.0, 1.0, /* 2 */
  1.0, 0.0, /* 1 */
  1.0, 1.0, /* 3 */
  0.0, 1.0, /* 2 */

  // BACK
  0.0, 0.0, /* 6 */
  0.0, 1.0, /* 4 */
  1.0, 1.0, /* 5 */
  0.0, 0.0, /* 6 */
  1.0, 1.0, /* 5 */
  1.0, 0.0, /* 7 */

  // TOP
  0.0, 1.0, /* 2 */
  1.0, 1.0, /* 3 */
  1.0, 1.0, /* 5 */
  0.0, 1.0, /* 2 */
  1.0, 1.0, /* 5 */
  0.0, 1.0, /* 4 */

  // BOTTOM
  0.0, 0.0, /* 0 */
  1.0, 0.0, /* 1 */
  1.0, 0.0, /* 7 */
  0.0, 0.0, /* 0 */
  1.0, 0.0, /* 7 */
  0.0, 0.0, /* 6 */

  // LEFT
  0.0, 0.0, /* 6 */
  0.0, 0.0, /* 0 */
  0.0, 1.0, /* 2 */
  0.0, 0.0, /* 6 */
  0.0, 1.0, /* 2 */
  0.0, 1.0, /* 4 */

  // RIGHT
  1.0, 0.0, /* 1 */
  1.0, 0.0, /* 7 */
  1.0, 1.0, /* 5 */
  1.0, 0.0, /* 1 */
  1.0, 1.0, /* 5 */
  1.0, 1.0, /* 3 */
};

#endif /* __3d_h */

